﻿/*
 * YBTouchPadSetting
 * (C)2017 John Doe
 *
 * This software is released under the MIT License.
 * http://opensource.org/licenses/mit-license.php
 */

using System.Linq;
using System.Windows;

namespace YBTouchPadSetting
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        //タッチパッドエリア設定用定数
        const string REG_KEY_TOUCH_DRIVER = @"SYSTEM\CurrentControlSet\Services\GoodixTouchDriver";

        const string REG_TP_TOP = @"TP_TOP";
        const string REG_TP_BOTTOM = @"TP_BOTTOM";
        const string REG_TP_LEFT = @"TP_LEFT";
        const string REG_TP_RIGHT = @"TP_RIGHT";

        const string REG_TP_A_RIGHT = @"TP_A_RIGHT";
        const string REG_TP_B_LEFT = @"TP_B_LEFT";

        const string REG_TP_D_TOP = @"TP_D_TOP";
        const string REG_TP_D_LEFT = @"TP_D_LEFT";

        const string REG_TP_E_TOP = @"TP_E_TOP";
        const string REG_TP_E_BOTTOM = @"TP_E_BOTTOM";
        const string REG_TP_E_LEFT = @"TP_E_LEFT";
        const string REG_TP_E_RIGHT = @"TP_E_RIGHT";

        const int DEFAULT_D_TOP = 119;
        const int DEFAULT_D_LEFT = 892;

        const int DEFAULT_E_TOP = 217;
        const int DEFAULT_E_BOTTOM = 121;
        const int DEFAULT_E_LEFT = 844;
        const int DEFAULT_E_RIGHT = 937;

        const int SMALL_D_TOP = 59;
        const int SMALL_D_LEFT = 900;

        const int MIDDLE_E_TOP = 270;

        //マウスポインタ設定用定数
        const string REG_KEY_MOUSE = @"Control Panel\Mouse";

        const string REG_SMOOTH_MOUSE_X_CURVE = @"SmoothMouseXCurve";
        const string REG_SMOOTH_MOUSE_Y_CURVE = @"SmoothMouseYCurve";

        readonly byte[] defaultXCurve = {
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x15, 0x6e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x40, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x29, 0xdc, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x00};
        readonly byte[] defaultYCurve = {
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0xb8, 0x5e, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
            0xcd, 0x4c, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00,
            0xcd, 0x4c, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x38, 0x02, 0x00, 0x00, 0x00, 0x00};
        readonly byte[] accelXCurveM = {
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x15, 0x6e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x80, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
            0xcc, 0xcc, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00};
        readonly byte[] accelYCurveM = {
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0xb8, 0x5e, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x3c, 0x02, 0x00, 0x00, 0x00, 0x00};
        readonly byte[] accelXCurveL = {
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x15, 0x6e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x66, 0xe6, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00};
        readonly byte[] accelYCurveL = {
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0xb8, 0x5e, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00};

        //前の状態を保持するための変数
        int prevRClickMode;     //1:large(default), 2:small, 3:none
        int prevUnlockMode;     //1:small(default), 2:middle, 3:large
        int prevAccelMode;      //1:normal(default), 2:middle, 3:large

        public MainWindow()
        {
            InitializeComponent();

            if (!getRegistry())
            {
                MessageBox.Show(Properties.Resources.MES_REG_NOT_FOUND, Properties.Resources.TITLE_ERROR, MessageBoxButton.OK, MessageBoxImage.Error);
                this.Close();
            }

            if (prevRClickMode == 3) this.RClickButton3.IsChecked = true;
            else if (prevRClickMode == 2) this.RClickButton2.IsChecked = true;
            else this.RClickButton1.IsChecked = true;

            if (prevUnlockMode == 3) this.UnlockButton3.IsChecked = true;
            else if (prevUnlockMode == 2) this.UnlockButton2.IsChecked = true;
            else this.UnlockButton1.IsChecked = true;

            if (prevAccelMode == 3) this.AccelButton3.IsChecked = true;
            else if (prevAccelMode == 2) this.AccelButton2.IsChecked = true;
            else this.AccelButton1.IsChecked = true;
        }

        private void okButton_Click(object sender, RoutedEventArgs e)
        {
            bool noerr = true;
            bool needReboot = false;
            int newRClickMode;
            int newUnlockMode;
            int newAccelMode;

            if ((bool)this.RClickButton3.IsChecked) newRClickMode = 3;
            else if ((bool)this.RClickButton2.IsChecked) newRClickMode = 2;
            else newRClickMode = 1;

            if ((bool)this.UnlockButton3.IsChecked) newUnlockMode = 3;
            else if ((bool)this.UnlockButton2.IsChecked) newUnlockMode = 2;
            else newUnlockMode = 1;

            if ((bool)this.AccelButton3.IsChecked) newAccelMode = 3;
            else if ((bool)this.AccelButton2.IsChecked) newAccelMode = 2;
            else newAccelMode = 1;

            if (newRClickMode != prevRClickMode || newUnlockMode != prevUnlockMode)
            {
                needReboot = true;
                noerr = setTouchDriverRegistry(newRClickMode, newUnlockMode);
            }
            if (noerr && newAccelMode != prevAccelMode)
            {
                needReboot = true;
                noerr = setMouseRegistry(newAccelMode);
            }

            if (noerr)
            {
                if (needReboot)
                {
                    MessageBox.Show(Properties.Resources.MES_REG_WRITE_OK, Properties.Resources.TITLE_INFO, MessageBoxButton.OK, MessageBoxImage.Information);
                }
            }
            else
            {
                MessageBox.Show(Properties.Resources.MES_REG_WRITE_ERROR, Properties.Resources.TITLE_ERROR, MessageBoxButton.OK, MessageBoxImage.Error);
            }
            this.Close();
        }

        private void cancelButton_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private bool getRegistry()
        {
            bool noerr = false;
            using (Microsoft.Win32.RegistryKey regKey = Microsoft.Win32.Registry.LocalMachine.OpenSubKey(REG_KEY_TOUCH_DRIVER))
            {
                try
                {
                    if (regKey != null)
                    {
                        int valueDTop = (int)regKey.GetValue(REG_TP_D_TOP);
                        int valueETop = (int)regKey.GetValue(REG_TP_E_TOP);

                        if (valueDTop == DEFAULT_D_TOP) prevRClickMode = 1;
                        else if (valueDTop == SMALL_D_TOP) prevRClickMode = 2;
                        else prevRClickMode = 3;

                        if (valueETop == DEFAULT_E_TOP) prevUnlockMode = 1;
                        else if (valueETop == MIDDLE_E_TOP) prevUnlockMode = 2;
                        else prevUnlockMode = 3;

                        noerr = true;
                    }
                }
                catch
                {
                    noerr = false;
                }
            }
            if (!noerr) return noerr;
            using (Microsoft.Win32.RegistryKey regKey = Microsoft.Win32.Registry.CurrentUser.OpenSubKey(REG_KEY_MOUSE))
            {
                try
                {
                    if (regKey != null)
                    {
                        byte[] accelMouseX = (byte[])regKey.GetValue(REG_SMOOTH_MOUSE_X_CURVE);

                        if (accelMouseX.SequenceEqual(defaultXCurve)) prevAccelMode = 1;
                        else if (accelMouseX.SequenceEqual(accelXCurveM)) prevAccelMode = 2;
                        else prevAccelMode = 3;

                        noerr = true;
                    }
                }
                catch
                {
                    noerr = false;
                }
            }
            return noerr;
        }

        private bool setTouchDriverRegistry(int rcMode, int ulMode)
        {
            bool noerr = false;
            using (Microsoft.Win32.RegistryKey regKey = Microsoft.Win32.Registry.LocalMachine.OpenSubKey(REG_KEY_TOUCH_DRIVER, true))
            {
                try
                {
                    if (regKey != null)
                    {
                        int valueTop = (int)regKey.GetValue(REG_TP_TOP);
                        int valueBottom = (int)regKey.GetValue(REG_TP_BOTTOM);
                        int valueARight = (int)regKey.GetValue(REG_TP_A_RIGHT);
                        int valueBLeft = (int)regKey.GetValue(REG_TP_B_LEFT);

                        int valueDTop = DEFAULT_D_TOP;
                        int valueDLeft = DEFAULT_D_LEFT;

                        int valueETop = DEFAULT_E_TOP;
                        int valueEBottom = DEFAULT_E_BOTTOM;
                        int valueELeft = DEFAULT_E_LEFT;
                        int valueERight = DEFAULT_E_RIGHT;

                        switch (rcMode)
                        {
                            case 3:
                                valueDTop = valueBottom;
                                valueDLeft = valueBLeft;
                                break;
                            case 2:
                                valueDTop = SMALL_D_TOP;
                                valueDLeft = SMALL_D_LEFT;
                                break;
                            default:
                                break;
                        }
                        regKey.SetValue(REG_TP_D_TOP, valueDTop);
                        regKey.SetValue(REG_TP_D_LEFT, valueDLeft);

                        switch (ulMode)
                        {
                            case 3:
                                valueETop = valueTop;
                                valueEBottom = valueBottom;
                                valueELeft = valueARight;
                                valueERight = valueBLeft;
                                break;
                            case 2:
                                valueETop = MIDDLE_E_TOP;
                                valueEBottom = valueBottom;
                                valueELeft = valueARight;
                                valueERight = valueBLeft;
                                break;
                            default:
                                break;
                        }
                        regKey.SetValue(REG_TP_E_TOP, valueETop);
                        regKey.SetValue(REG_TP_E_BOTTOM, valueEBottom);
                        regKey.SetValue(REG_TP_E_LEFT, valueELeft);
                        regKey.SetValue(REG_TP_E_RIGHT, valueERight);

                        noerr = true;
                    }
                }
                catch
                {
                    noerr = false;
                }
            }
            return noerr;
        }

        private bool setMouseRegistry(int accelMode)
        {
            bool noerr = false;
            using (Microsoft.Win32.RegistryKey regKey = Microsoft.Win32.Registry.CurrentUser.OpenSubKey(REG_KEY_MOUSE, true))
            {
                try
                {
                    if (regKey != null)
                    {
                        switch (accelMode)
                        {
                            case 3:
                                regKey.SetValue(REG_SMOOTH_MOUSE_X_CURVE, accelXCurveL);
                                regKey.SetValue(REG_SMOOTH_MOUSE_Y_CURVE, accelYCurveL);
                                break;
                            case 2:
                                regKey.SetValue(REG_SMOOTH_MOUSE_X_CURVE, accelXCurveM);
                                regKey.SetValue(REG_SMOOTH_MOUSE_Y_CURVE, accelYCurveM);
                                break;
                            default:
                                regKey.SetValue(REG_SMOOTH_MOUSE_X_CURVE, defaultXCurve);
                                regKey.SetValue(REG_SMOOTH_MOUSE_Y_CURVE, defaultYCurve);
                                break;
                        }
                        noerr = true;
                    }
                }
                catch
                {
                    noerr = false;
                }
            }
            return noerr;
        }

    }
}